<?php
/*
Plugin Name: Intenal URLs
Version: 2.0
Plugin URI: http://www.pierovdfn.it/redirect/plugin_internalurls.html
Description: Allows using the iurl bbcode tag to make internal urls
Author: Piero VDFN
Author URI: http://www.pierovdfn.it/
*/

/**
 * The class to manage those custom tags.
 */
class Plugin_InternalURLs {

	/**
	 * The constructor. Set hook.
	 */
	public function __construct()
	{
		add_filter('bbcode_init', array($this, 'setTags'), 10, 1);
	}

	/**
	 * Tells the BBCode plugin about the tags of this plugin.
	 *
	 * @param object $bbcode: The BBCode instance
	 * @return object: The first parameter
	 */
	public function setTags($bbcode)
	{
		// iurl tag
		$bbcode->addCode(
			'iurl', 
			'callback_replace', 
			array($this, 'doIurl'), 
			array(
				'usecontent_param' => array('default', 'new')
			),
			'link',
			array(
				'listitem', 'block', 'inline'
			),
			array('link')
		);
		$bbcode->setCodeFlag('iurl', 'closetag', BBCODE_CLOSETAG_OPTIONAL);

		// entryurl tag
		$bbcode->addCode(
			'entryurl', 
			'callback_replace', 
			array($this, 'doEntryUrl'),
			array(
				'usecontent_param' => array('default', 'new')
			),
			'link',
			array(
				'listitem', 'block', 'inline'
			),
			array('link')
		);
		$bbcode->setCodeFlag('entryurl', 'closetag', BBCODE_CLOSETAG_OPTIONAL);	

		return $bbcode;
	}

	/**
	 * Handle the iurl tag.
	 *
	 * @param string $action: The action to do
	 * @param array $attributes: The attributes of the tag
	 * @param string $content: The tag content
	 * @param ??? $params: Unknown
	 * @param ??? $node_object: Unknown
	 */
	public function doIurl($action, $attributes, $content, $params, $node_object)
	{
		if($action == 'validate') {
			return true;
		}

		if(!isset($attributes['default'])) {
			return '[No valid url specified]';
		}

		$url = explode(':', $attributes['default'], 2);
		switch($url[0]) {
			case 'index':
				$the_url = BLOG_BASEURL;
			break;

			case 'contact':
				$the_url = BLOG_BASEURL.'contact.php';
			break;

			case 'static':
			case 'page':
				$the_url = theme_staticlink($url[1]);
			break;

			case 'post':
			case 'entry':
				$e = $url[1];

				// Make sure it begins with entry
				if(substr($e, 0, 5)!='entry') {
					$e = 'entry'.$e;
				}

				// Pretty URLs Hack
				if(class_exists('Plugin_PrettyURLs')) {
					global $post;
					$fpdb = new FPDB();
					$params = array('id' => $e, 'fullparse' => false, 'comments' => 'false');
					$fpdb->query($params);

					if(!is_array($fpdb->queries) || !count($fpdb->queries)) {
						return '[No valid url specified]';
					}

					$q = $fpdb->getQuery(key($fpdb->queries));
					@$op = $post;
					list($e, $post) = @$q->peekEntry();
					unset($q);
					unset($fpdb);
				}


				if(@$attributes['comments']) {
					$the_url = get_comments_link($e);
				} else {
					$the_url = get_permalink($e);
				}

				if(isset($post)) {
					$post = $op;
				}

			break;

			case 'category':
			case 'cat':
				$the_url = get_category_link($url[1]);
			break;

			case 'tag':
				if(class_exists('plugin_tag')) {
					$the_url = apply_filters('tag_link', $url[1]); # Tag plugin >= 1.1 
					break;
				}
			// No break: return the parameter

			case 'redirect':
				if(class_exists('plugin_redirect')) {
					global $plugin_redirect;
					if(is_callable(array(&$plugin_redirect, 'redirect_url'))) {
						$the_url = $plugin_redirect->redirect_url($url[1]);
						break;
					}
				}
			// No break: return the parameter

			case 'relative':
				$the_url = BLOG_BASEURL.$url[1];
			break;

			// Return the orignial parameter
			default:
				$the_url = $attributes['default'];
			break;
		}
 
		$rel = isset($attributes['rel']) ? ' rel="' . $attributes['rel'] . '"' : '';
		$title = isset($attributes['title']) ?  ' title="' . $attributes['title'] . '"' : '';
		$the_url = isset($attributes['anchor']) ? $the_url . '#' . $attributes['anchor'] : $the_url;

		if(isset($attributes['onlyurl'])) {
			return $the_url;
		}

   		return '<a href="' . $the_url . '"' . $rel . $title . '>' . $content . '</a>';
	}

	/**
	 * Handle the entryurl tag.
	 *
	 * @param string $action: The action to do
	 * @param array $attributes: The attributes of the tag
	 * @param string $content: The tag content
	 * @param ??? $params: Unknown
	 * @param ??? $node_object: Unknown
	 */
	public function doEntryUrl($action, $attributes, $content, $params, $node_object)
	{
		$entry = @$attributes['default'];

		if(empty($entry)) {
			return '[No valid url specified]';
		}

		// Make sure it begins with entry
		if(substr($entry, 0, 5) != 'entry') {
			$entry = 'entry' . $entry;
		}

		global $post;
		$fpdb = new FPDB();
		$params = array('id' => $entry, 'fullparse' => false, 'comments' => 'false');
		$fpdb->query($params);

		if(!is_array($fpdb->queries) || !count($fpdb->queries)) {
			return '[No valid url specified]';
		}

		$q = $fpdb->getQuery(key($fpdb->queries));
		@$op = $post;
		list($entry, $post) = @$q->peekEntry();
		unset($q);
		unset($fpdb);

		if(empty($content) || is_null($content)) {
			$content = $post['subject'];
		}

		if(@$attributes['comments']) {
			$url = get_comments_link($entry);
		} else {
			$url = get_permalink($entry);
		}

		$post = $op;

		$rel = isset($attributes['rel']) ? ' rel="' . $attributes['rel'] . '"' : '';
		$title = isset($attributes['title']) ?  ' title="' . $attributes['title'] . '"' : '';
		$url = isset($attributes['anchor']) ? $url . '#' . $attributes['anchor'] : $url;

		return '<a href="' . $url . '"' . $rel . $title . '>' . $content . '</a>';
	}

}

new Plugin_InternalURLs();
